@@ -185,6 +185,9 @@ open_gpgolconfig (LPVOID arg)
   args.push_back (std::string("--gpgol-version"));
   args.push_back (std::string(VERSION));
 
+  args.push_back (std::string ("--lang"));
+  args.push_back (std::string (gettext_localename ()));
+
   auto ctx = GpgME::Context::createForEngine (GpgME::SpawnEngine);
   if (!ctx)
     {
