@@ -48,8 +48,10 @@ std::vector <std::string> cArray_to_vector (const char **cArray);
 /* Check if we are in de_vs mode. */
 bool in_de_vs_mode ();
 
+#ifdef HAVE_W32_SYSTEM
 /* Get a map of all subkey value pairs in a registry key */
 std::map<std::string, std::string> get_registry_subkeys (const char *path);
+#endif
 
 std::vector<std::string> gpgol_split (const std::string &s, char delim);
 
