@@ -39,14 +39,13 @@
 
 #include <sstream>
 
-#define DEBUG_RESOLVER 1
-
 static int
 sink_data_write (sink_t sink, const void *data, size_t datalen)
 {
+  TSTART;
   GpgME::Data *d = static_cast<GpgME::Data *>(sink->cb_data);
   d->write (data, datalen);
-  return 0;
+  TRETURN 0;
 }
 
 static int
@@ -68,22 +67,27 @@ CryptController::CryptController (Mail *mail, bool encrypt, bool sign,
     m_crypto_success (false),
     m_proto (proto)
 {
+  TSTART;
   memdbg_ctor ("CryptController");
   log_debug ("%s:%s: CryptController ctor for %p encrypt %i sign %i inline %i.",
              SRCNAME, __func__, mail, encrypt, sign, mail->getDoPGPInline ());
   m_recipient_addrs = mail->getCachedRecipients ();
+  TRETURN;
 }
 
 CryptController::~CryptController()
 {
+  TSTART;
   memdbg_dtor ("CryptController");
   log_debug ("%s:%s:%p",
              SRCNAME, __func__, m_mail);
+  TRETURN;
 }
 
 int
 CryptController::collect_data ()
 {
+  TSTART;
   /* Get the attachment info and the body.  We need to do this before
      creating the engine's filter because sending the cancel to
      the engine with nothing for the engine to process.  Will result
@@ -119,7 +123,7 @@ CryptController::collect_data ()
       gpgol_release (message);
       mapi_release_attach_table (att_table);
       xfree (body);
-      return -1;
+      TRETURN -1;
     }
 
   bool do_inline = m_mail->getDoPGPInline ();
@@ -162,7 +166,7 @@ CryptController::collect_data ()
                  SRCNAME, __func__);
       gpgol_release (message);
       mapi_release_attach_table (att_table);
-      return -1;
+      TRETURN -1;
     }
 
   /* Message is no longer needed */
@@ -172,13 +176,14 @@ CryptController::collect_data ()
 
   /* Set the input buffer to start. */
   m_input.seek (0, SEEK_SET);
-  return 0;
+  TRETURN 0;
 }
 
 int
 CryptController::lookup_fingerprints (const std::string &sigFpr,
                                       const std::vector<std::string> recpFprs)
 {
+  TSTART;
   auto ctx = std::shared_ptr<GpgME::Context> (GpgME::Context::createForProtocol (m_proto));
 
   if (!ctx)
@@ -188,7 +193,7 @@ CryptController::lookup_fingerprints (const std::string &sigFpr,
                  m_proto == GpgME::CMS ? "smime" :
                  m_proto == GpgME::OpenPGP ? "openpgp" :
                  "unknown");
-      return -1;
+      TRETURN -1;
     }
 
   ctx->setKeyListMode (GpgME::Local);
@@ -198,11 +203,11 @@ CryptController::lookup_fingerprints (const std::string &sigFpr,
       m_signer_key = ctx->key (sigFpr.c_str (), err, true);
       if (err || m_signer_key.isNull ()) {
           log_error ("%s:%s: failed to lookup key for '%s' with protocol '%s'",
-                     SRCNAME, __func__, sigFpr.c_str (),
+                     SRCNAME, __func__, anonstr (sigFpr.c_str ()),
                      m_proto == GpgME::CMS ? "smime" :
                      m_proto == GpgME::OpenPGP ? "openpgp" :
                      "unknown");
-          return -1;
+          TRETURN -1;
       }
       // reset context
       ctx = std::shared_ptr<GpgME::Context> (GpgME::Context::createForProtocol (m_proto));
@@ -210,7 +215,7 @@ CryptController::lookup_fingerprints (const std::string &sigFpr,
   }
 
   if (!recpFprs.size()) {
-      return 0;
+     TRETURN 0;
   }
 
   // Convert recipient fingerprints
@@ -222,7 +227,7 @@ CryptController::lookup_fingerprints (const std::string &sigFpr,
       log_error ("%s:%s: failed to start recipient keylisting",
                  SRCNAME, __func__);
       release_cArray (cRecps);
-      return -1;
+      TRETURN -1;
   }
 
   do {
@@ -233,13 +238,14 @@ CryptController::lookup_fingerprints (const std::string &sigFpr,
 
   release_cArray (cRecps);
 
-  return 0;
+  TRETURN 0;
 }
 
 
 int
 CryptController::parse_output (GpgME::Data &resolverOutput)
 {
+  TSTART;
   // Todo: Use Data::toString
   std::istringstream ss(resolverOutput.toString());
   std::string line;
@@ -253,13 +259,13 @@ CryptController::parse_output (GpgME::Data &resolverOutput)
         {
           log_debug ("%s:%s: resolver canceled",
                      SRCNAME, __func__);
-          return -2;
+          TRETURN -2;
         }
       if (line == "unencrypted")
         {
           log_debug ("%s:%s: FIXME resolver wants unencrypted",
                      SRCNAME, __func__);
-          return -1;
+          TRETURN -1;
         }
       std::istringstream lss (line);
 
@@ -306,10 +312,10 @@ CryptController::parse_output (GpgME::Data &resolverOutput)
                  SRCNAME, __func__);
       gpgol_message_box (m_mail->getWindow (), _("No recipients for encryption selected."),
                          _("GpgOL"), MB_OK);
-      return -2;
+      TRETURN -2;
     }
 
-  return lookup_fingerprints (sigFpr, recpFprs);
+  TRETURN lookup_fingerprints (sigFpr, recpFprs);
 }
 
 static bool
@@ -319,6 +325,7 @@ resolve_through_protocol (const GpgME::Protocol &proto, bool sign,
                           std::vector<GpgME::Key> &r_keys,
                           GpgME::Key &r_sig)
 {
+  TSTART;
   bool sig_ok = true;
   bool enc_ok = true;
 
@@ -334,12 +341,13 @@ resolve_through_protocol (const GpgME::Protocol &proto, bool sign,
       r_sig = cache->getSigningKey (sender.c_str (), proto);
       sig_ok = !r_sig.isNull();
     }
-  return sig_ok && enc_ok;
+  TRETURN sig_ok && enc_ok;
 }
 
 int
 CryptController::resolve_keys_cached()
 {
+  TSTART;
   // Prepare variables
   const auto cached_sender = m_mail->getSender ();
   auto recps = m_recipient_addrs;
@@ -394,7 +402,7 @@ CryptController::resolve_keys_cached()
       m_recipients.clear();
       m_signer_key = GpgME::Key();
       m_proto = GpgME::UnknownProtocol;
-      return 1;
+      TRETURN 1;
     }
 
   if (!m_recipients.empty())
@@ -404,28 +412,46 @@ CryptController::resolve_keys_cached()
     }
   for (const auto &key: m_recipients)
     {
-      log_debug ("%s", key.primaryFingerprint ());
+      log_debug ("%s", anonstr (key.primaryFingerprint ()));
     }
   if (!m_signer_key.isNull())
     {
       log_debug ("%s:%s: Signing key: %s:%s",
-                 SRCNAME, __func__, m_signer_key.primaryFingerprint (),
+                 SRCNAME, __func__, anonstr (m_signer_key.primaryFingerprint ()),
                  to_cstr (m_signer_key.protocol()));
     }
-  return 0;
+  TRETURN 0;
 }
 
 int
 CryptController::resolve_keys ()
 {
+  TSTART;
   m_recipients.clear();
 
+  if (!m_recipient_addrs.size())
+    {
+      /* Should not happen. But we add it for better bug reports. */
+      const char *bugmsg = utf8_gettext ("Operation failed.\n\n"
+              "This is usually caused by a bug in GpgOL or an error in your setup.\n"
+              "Please see https://www.gpg4win.org/reporting-bugs.html "
+              "or ask your Administrator for support.");
+      char *buf;
+      gpgrt_asprintf (&buf, "Failed to resolve recipients.\n\n%s\n", bugmsg);
+      memdbg_alloc (buf);
+      gpgol_message_box (get_active_hwnd (),
+                         buf,
+                         _("GpgOL"), MB_OK);
+      xfree(buf);
+      TRETURN -1;
+    }
+
   if (opt.autoresolve && !resolve_keys_cached ())
     {
       log_debug ("%s:%s: resolved keys through the cache",
                  SRCNAME, __func__);
       start_crypto_overlay();
-      return 0;
+      TRETURN 0;
     }
 
   std::vector<std::string> args;
@@ -435,7 +461,7 @@ CryptController::resolve_keys ()
   if (!gpg4win_dir)
     {
       TRACEPOINT;
-      return -1;
+      TRETURN -1;
     }
   const auto resolver = std::string (gpg4win_dir) + "\\bin\\resolver.exe";
   args.push_back (resolver);
@@ -493,6 +519,14 @@ CryptController::resolve_keys ()
       args.push_back (std::string ("--alwaysShow"));
     }
 
+  if (opt.prefer_smime)
+    {
+      args.push_back (std::string ("--preferred-protocol"));
+      args.push_back (std::string ("cms"));
+    }
+
+  args.push_back (std::string ("--lang"));
+  args.push_back (std::string (gettext_localename ()));
 
   if (m_encrypt)
     {
@@ -500,20 +534,36 @@ CryptController::resolve_keys ()
       // Get the recipients that are cached from OOM
       for (const auto &addr: m_recipient_addrs)
         {
-          args.push_back (GpgME::UserID::addrSpecFromString (addr.c_str()));
+          const auto mbox = GpgME::UserID::addrSpecFromString (addr.c_str());
+          const auto overrides = KeyCache::instance ()->getOverrides (mbox);
+          if (overrides.size())
+            {
+              std::string overrideStr = mbox + ":";
+              for (const auto &key: overrides)
+                {
+                  if (key.isNull())
+                    {
+                      TRACEPOINT;
+                      continue;
+                    }
+                  overrideStr += key.primaryFingerprint();
+                  overrideStr += ",";
+                }
+              overrideStr.erase(overrideStr.size() - 1, 1);
+              args.push_back (std::string ("-o"));
+              args.push_back (overrideStr);
+            }
+          args.push_back (mbox);
         }
     }
 
-  args.push_back (std::string ("--lang"));
-  args.push_back (std::string (gettext_localename ()));
-
   // Args are prepared. Spawn the resolver.
   auto ctx = GpgME::Context::createForEngine (GpgME::SpawnEngine);
   if (!ctx)
     {
       // can't happen
       TRACEPOINT;
-      return -1;
+      TRETURN -1;
     }
 
 
@@ -521,13 +571,12 @@ CryptController::resolve_keys ()
   // It's a bit overhead but should be quick for such small
   // data.
   char **cargs = vector_to_cArray (args);
-#ifdef DEBUG_RESOLVER
-  log_debug ("Spawning args:");
+  log_data ("%s:%s: Spawn args:",
+            SRCNAME, __func__);
   for (size_t i = 0; cargs && cargs[i]; i++)
     {
-      log_debug (SIZE_T_FORMAT ": '%s'", i, cargs[i]);
+      log_data (SIZE_T_FORMAT ": '%s'", i, cargs[i]);
     }
-#endif
 
   GpgME::Data mystdin (GpgME::Data::null), mystdout, mystderr;
   GpgME::Error err = ctx->spawn (cargs[0], const_cast <const char**> (cargs),
@@ -542,10 +591,8 @@ CryptController::resolve_keys ()
   // We need to create an overlay while encrypting as pinentry can take a while
   start_crypto_overlay();
 
-#ifdef DEBUG_RESOLVER
-  log_debug ("Resolver stdout:\n'%s'", mystdout.toString ().c_str ());
-  log_debug ("Resolver stderr:\n'%s'", mystderr.toString ().c_str ());
-#endif
+  log_data ("Resolver stdout:\n'%s'", mystdout.toString ().c_str ());
+  log_data ("Resolver stderr:\n'%s'", mystderr.toString ().c_str ());
 
   release_cArray (cargs);
 
@@ -560,17 +607,18 @@ CryptController::resolve_keys ()
     {
       log_debug ("%s:%s: Failed to parse / resolve keys.",
                  SRCNAME, __func__);
-      log_debug ("Resolver stdout:\n'%s'", mystdout.toString ().c_str ());
-      log_debug ("Resolver stderr:\n'%s'", mystderr.toString ().c_str ());
-      return -1;
+      log_data ("Resolver stdout:\n'%s'", mystdout.toString ().c_str ());
+      log_data ("Resolver stderr:\n'%s'", mystderr.toString ().c_str ());
+      TRETURN -1;
     }
 
-  return ret;
+  TRETURN ret;
 }
 
 int
 CryptController::do_crypto (GpgME::Error &err)
 {
+  TSTART;
   log_debug ("%s:%s",
              SRCNAME, __func__);
 
@@ -583,12 +631,12 @@ CryptController::do_crypto (GpgME::Error &err)
       //error
       log_debug ("%s:%s: Failure to resolve keys.",
                  SRCNAME, __func__);
-      return -1;
+      TRETURN -1;
     }
   if (ret == -2)
     {
       // Cancel
-      return -2;
+      TRETURN -2;
     }
   bool do_inline = m_mail->getDoPGPInline ();
 
@@ -610,7 +658,7 @@ CryptController::do_crypto (GpgME::Error &err)
       gpgol_message_box (m_mail->getWindow (),
                          "Failure to create context.",
                          utf8_gettext ("GpgOL"), MB_OK);
-      return -1;
+      TRETURN -1;
     }
   if (!m_signer_key.isNull())
     {
@@ -636,7 +684,7 @@ CryptController::do_crypto (GpgME::Error &err)
                      SRCNAME, __func__, result_pair.first.error().asString(),
                      result_pair.second.error().asString());
           err = err1 ? err1 : err2;
-          return -1;
+          TRETURN -1;
         }
 
       if (err1.isCanceled() || err2.isCanceled())
@@ -644,7 +692,7 @@ CryptController::do_crypto (GpgME::Error &err)
           err = err1.isCanceled() ? err1 : err2;
           log_debug ("%s:%s: User cancled",
                      SRCNAME, __func__);
-          return -2;
+          TRETURN -2;
         }
     }
   else if (m_encrypt && m_sign)
@@ -657,13 +705,13 @@ CryptController::do_crypto (GpgME::Error &err)
         {
           log_error ("%s:%s: Signing error %s.",
                      SRCNAME, __func__, sigResult.error().asString());
-          return -1;
+          TRETURN -1;
         }
       if (err.isCanceled())
         {
           log_debug ("%s:%s: User cancled",
                      SRCNAME, __func__);
-          return -2;
+          TRETURN -2;
         }
       parse_micalg (sigResult);
 
@@ -684,7 +732,7 @@ CryptController::do_crypto (GpgME::Error &err)
                               m_output, m_input, m_micalg.c_str ()))
         {
           TRACEPOINT;
-          return -1;
+          TRETURN -1;
         }
 
       // Now we have the multipart throw away the rest.
@@ -699,13 +747,13 @@ CryptController::do_crypto (GpgME::Error &err)
         {
           log_error ("%s:%s: Encryption error %s.",
                      SRCNAME, __func__, err.asString());
-          return -1;
+          TRETURN -1;
         }
       if (err.isCanceled())
         {
           log_debug ("%s:%s: User cancled",
                      SRCNAME, __func__);
-          return -2;
+          TRETURN -2;
         }
       // Now we have encrypted output just treat it like encrypted.
     }
@@ -719,13 +767,13 @@ CryptController::do_crypto (GpgME::Error &err)
         {
           log_error ("%s:%s: Encryption error %s.",
                      SRCNAME, __func__, err.asString());
-          return -1;
+          TRETURN -1;
         }
       if (err.isCanceled())
         {
           log_debug ("%s:%s: User cancled",
                      SRCNAME, __func__);
-          return -2;
+          TRETURN -2;
         }
     }
   else if (m_sign)
@@ -738,13 +786,13 @@ CryptController::do_crypto (GpgME::Error &err)
         {
           log_error ("%s:%s: Signing error %s.",
                      SRCNAME, __func__, err.asString());
-          return -1;
+          TRETURN -1;
         }
       if (err.isCanceled())
         {
           log_debug ("%s:%s: User cancled",
                      SRCNAME, __func__);
-          return -2;
+          TRETURN -2;
         }
       parse_micalg (result);
     }
@@ -760,15 +808,16 @@ CryptController::do_crypto (GpgME::Error &err)
              SRCNAME, __func__);
   m_crypto_success = true;
 
-  return 0;
+  TRETURN 0;
 }
 
 static int
 write_data (sink_t sink, GpgME::Data &data)
 {
+  TSTART;
   if (!sink || !sink->writefnc)
     {
-      return -1;
+      TRETURN -1;
     }
 
   char buf[4096];
@@ -779,7 +828,7 @@ write_data (sink_t sink, GpgME::Data &data)
       sink->writefnc (sink, buf, nread);
     }
 
-  return 0;
+  TRETURN 0;
 }
 
 int
@@ -788,6 +837,7 @@ create_sign_attach (sink_t sink, protocol_t protocol,
                     GpgME::Data &signedData,
                     const char *micalg)
 {
+  TSTART;
   char boundary[BOUNDARYSIZE+1];
   char top_header[BOUNDARYSIZE+200];
   int rc = 0;
@@ -801,28 +851,28 @@ create_sign_attach (sink_t sink, protocol_t protocol,
   if ((rc = write_string (sink, top_header)))
     {
       TRACEPOINT;
-      return rc;
+      TRETURN rc;
     }
 
   /* Write the boundary so that it is not included in the hashing.  */
   if ((rc = write_boundary (sink, boundary, 0)))
     {
       TRACEPOINT;
-      return rc;
+      TRETURN rc;
     }
 
   /* Write the signed mime structure */
   if ((rc = write_data (sink, signedData)))
     {
       TRACEPOINT;
-      return rc;
+      TRETURN rc;
     }
 
   /* Write the signature attachment */
   if ((rc = write_boundary (sink, boundary, 0)))
     {
       TRACEPOINT;
-      return rc;
+      TRETURN rc;
     }
 
   if (protocol == PROTOCOL_OPENPGP)
@@ -847,13 +897,13 @@ create_sign_attach (sink_t sink, protocol_t protocol,
   if (rc)
     {
       TRACEPOINT;
-      return rc;
+      TRETURN rc;
     }
 
   if ((rc = write_string (sink, "\r\n")))
     {
       TRACEPOINT;
-      return rc;
+      TRETURN rc;
     }
 
   // Write the signature data
@@ -863,30 +913,30 @@ create_sign_attach (sink_t sink, protocol_t protocol,
       if ((rc = write_b64 (sink, (const void *) sigStr.c_str (), sigStr.size())))
         {
           TRACEPOINT;
-          return rc;
+          TRETURN rc;
         }
     }
   else if ((rc = write_data (sink, signature)))
     {
       TRACEPOINT;
-      return rc;
+      TRETURN rc;
     }
 
   // Add an extra linefeed with should not harm.
   if ((rc = write_string (sink, "\r\n")))
     {
       TRACEPOINT;
-      return rc;
+      TRETURN rc;
     }
 
   /* Write the final boundary.  */
   if ((rc = write_boundary (sink, boundary, 1)))
     {
       TRACEPOINT;
-      return rc;
+      TRETURN rc;
     }
 
-  return rc;
+  TRETURN rc;
 }
 
 static int
@@ -894,6 +944,7 @@ create_encrypt_attach (sink_t sink, protocol_t protocol,
                        GpgME::Data &encryptedData,
                        int exchange_major_version)
 {
+  TSTART;
   char boundary[BOUNDARYSIZE+1];
   int rc = create_top_encryption_header (sink, protocol, boundary,
                                          false, exchange_major_version);
@@ -902,7 +953,7 @@ create_encrypt_attach (sink_t sink, protocol_t protocol,
     {
       log_error ("%s:%s: Failed to create top header.",
                  SRCNAME, __func__);
-      return rc;
+      TRETURN rc;
     }
 
   if (protocol == PROTOCOL_OPENPGP ||
@@ -922,7 +973,7 @@ create_encrypt_attach (sink_t sink, protocol_t protocol,
     {
       log_error ("%s:%s: Failed to create top header.",
                  SRCNAME, __func__);
-      return rc;
+      TRETURN rc;
     }
 
   /* Write the final boundary (for OpenPGP) and finish the attachment.  */
@@ -931,12 +982,13 @@ create_encrypt_attach (sink_t sink, protocol_t protocol,
       log_error ("%s:%s: Failed to write boundary.",
                  SRCNAME, __func__);
     }
-  return rc;
+  TRETURN rc;
 }
 
 int
 CryptController::update_mail_mapi ()
 {
+  TSTART;
   log_debug ("%s:%s", SRCNAME, __func__);
 
   LPMESSAGE message = get_oom_base_message (m_mail->item());
@@ -944,7 +996,7 @@ CryptController::update_mail_mapi ()
     {
       log_error ("%s:%s: Failed to obtain message.",
                  SRCNAME, __func__);
-      return -1;
+      TRETURN -1;
     }
 
   if (m_mail->getDoPGPInline ())
@@ -962,7 +1014,7 @@ CryptController::update_mail_mapi ()
                      SRCNAME, __func__);
         }
 
-      return 0;
+      TRETURN 0;
     }
 
   mapi_attach_item_t *att_table = mapi_create_attach_table (message, 0);
@@ -999,7 +1051,7 @@ CryptController::update_mail_mapi ()
       log_error ("%s:%s: Failed to create moss attach.",
                  SRCNAME, __func__);
       gpgol_release (message);
-      return -1;
+      TRETURN -1;
     }
 
   protocol_t protocol = m_proto == GpgME::CMS ?
@@ -1050,16 +1102,17 @@ CryptController::update_mail_mapi ()
   gpgol_release (attach);
   gpgol_release (message);
 
-  return rc;
+  TRETURN rc;
 }
 
 std::string
 CryptController::get_inline_data ()
 {
+  TSTART;
   std::string ret;
   if (!m_mail->getDoPGPInline ())
     {
-      return ret;
+      TRETURN ret;
     }
   m_output.seek (0, SEEK_SET);
   char buf[4096];
@@ -1068,29 +1121,30 @@ CryptController::get_inline_data ()
     {
       ret += std::string (buf, nread);
     }
-  return ret;
+  TRETURN ret;
 }
 
 void
 CryptController::parse_micalg (const GpgME::SigningResult &result)
 {
+  TSTART;
   if (result.isNull())
     {
       TRACEPOINT;
-      return;
+      TRETURN;
     }
   const auto signature = result.createdSignature(0);
   if (signature.isNull())
     {
       TRACEPOINT;
-      return;
+      TRETURN;
     }
 
   const char *hashAlg = signature.hashAlgorithmAsString ();
   if (!hashAlg)
     {
       TRACEPOINT;
-      return;
+      TRETURN;
     }
   if (m_proto == GpgME::OpenPGP)
     {
@@ -1104,11 +1158,13 @@ CryptController::parse_micalg (const GpgME::SigningResult &result)
 
   log_debug ("%s:%s: micalg is: '%s'.",
              SRCNAME, __func__, m_micalg.c_str ());
+  TRETURN;
 }
 
 void
 CryptController::start_crypto_overlay ()
 {
+  TSTART;
   auto wid = m_mail->getWindow ();
 
   std::string text;
@@ -1122,4 +1178,5 @@ CryptController::start_crypto_overlay ()
       text = _("Signing...");
     }
   m_overlay = std::unique_ptr<Overlay> (new Overlay (wid, text));
+  TRETURN;
 }
