@@ -49,6 +49,7 @@
 #include "cpphelp.h"
 #include "dispcache.h"
 #include "categorymanager.h"
+#include "keycache.h"
 
 #include <gpg-error.h>
 #include <list>
@@ -524,6 +525,8 @@ GpgolAddin::OnStartupComplete (SAFEARRAY** custom)
 
   CloseHandle (CreateThread (NULL, 0, init_gpgme_config, nullptr, 0,
                              NULL));
+
+  KeyCache::instance ()->populate ();
   return S_OK;
 }
 
@@ -714,6 +717,7 @@ GpgolRibbonExtender::GetIDsOfNames (REFIID riid, LPOLESTR *rgszNames,
       ID_MAPPER (L"getIsCrypto", ID_GET_IS_CRYPTO_MAIL)
       ID_MAPPER (L"printDecrypted", ID_CMD_PRINT_DECRYPTED)
       ID_MAPPER (L"openContactKey", ID_CMD_OPEN_CONTACT_KEY)
+      ID_MAPPER (L"overrideFileClose", ID_CMD_FILE_CLOSE)
     }
 
   if (cNames > 1)
@@ -805,6 +809,8 @@ GpgolRibbonExtender::Invoke (DISPID dispid, REFIID riid, LCID lcid,
         return get_is_crypto_mail (parms->rgvarg[0].pdispVal, result);
       case ID_CMD_OPEN_CONTACT_KEY:
         return open_contact_key (parms->rgvarg[0].pdispVal);
+      case ID_CMD_FILE_CLOSE :
+        return override_file_close ();
       case ID_BTN_ENCRYPT:
       case ID_BTN_DECRYPT:
       case ID_BTN_DECRYPT_LARGE:
@@ -955,6 +961,10 @@ GetCustomUI_MIME (BSTR RibbonID, BSTR * RibbonXml)
       gpgrt_asprintf (&buffer,
         "<customUI xmlns=\"http://schemas.microsoft.com/office/2009/07/customui\""
         " onLoad=\"ribbonLoaded\">"
+        " <commands>"
+        "  <command idMso=\"FileCloseAndLogOff\""
+        "           onAction=\"overrideFileClose\"/>"
+        " </commands>"
         " <ribbon>"
         "   <tabs>"
         "    <tab idMso=\"TabMail\">"
@@ -1186,11 +1196,12 @@ GpgolAddin::addRibbon (LPDISPATCH disp)
 void
 GpgolAddin::shutdown ()
 {
+  TSTART;
   if (m_shutdown)
     {
       log_debug ("%s:%s: Already shutdown",
                  SRCNAME, __func__);
-      return;
+      TRETURN;
     }
 
   m_shutdown = true;
@@ -1230,6 +1241,7 @@ GpgolAddin::shutdown ()
 
   gpgol_release (m_application);
   m_application = nullptr;
+  TRETURN;
 }
 
 void
