@@ -32,6 +32,7 @@
 class GpgolAddinRibbonExt;
 class ApplicationEventListener;
 class DispCache;
+class CategoryManager;
 
 /* Enums for the IDTExtensibility2 interface*/
 typedef enum
@@ -217,6 +218,14 @@ public:
   std::shared_ptr<DispCache> get_dispcache () { return m_dispcache; }
   bool isShutdown() { return m_shutdown; };
 
+  /* Register a ribbon ui component */
+  void addRibbon (LPDISPATCH ribbon);
+
+  /* Invalidate the ribbons. */
+  void invalidateRibbons ();
+
+  std::shared_ptr<CategoryManager> get_category_mngr ();
+
 private:
   ULONG m_lRef;
   GpgolRibbonExtender* m_ribbonExtender;
@@ -231,6 +240,8 @@ private:
   HHOOK m_hook;
   std::vector<LPDISPATCH> m_explorerEventSinks;
   std::shared_ptr<DispCache> m_dispcache;
+  std::vector<LPDISPATCH> m_ribbon_uis;
+  std::shared_ptr<CategoryManager> m_category_mngr;
 };
 
 class GpgolAddinFactory: public IClassFactory
