@@ -43,7 +43,6 @@
 #endif
 
 
-
 const char decrypt_template_html[] = {
 "<html><head></head><body>"
 "<table border=\"0\" width=\"100%%\" cellspacing=\"1\" cellpadding=\"1\" bgcolor=\"#0069cc\">"
@@ -76,6 +75,39 @@ expect_no_mime (msgtype_t type)
          type == MSGTYPE_GPGOL_CLEAR_SIGNED;
 }
 
+#ifdef BUILD_TESTS
+static void
+get_and_print_key_test (const char *fingerprint, GpgME::Protocol proto)
+{
+  if (!fingerprint)
+    {
+      STRANGEPOINT;
+      return;
+    }
+  auto ctx = std::unique_ptr<GpgME::Context> (GpgME::Context::createForProtocol
+                                              (proto));
+
+  if (!ctx)
+    {
+      STRANGEPOINT;
+      return;
+    }
+  ctx->setKeyListMode (GpgME::KeyListMode::Local |
+                       GpgME::KeyListMode::Signatures |
+                       GpgME::KeyListMode::Validate |
+                       GpgME::KeyListMode::WithTofu);
+
+  GpgME::Error err;
+  const auto newKey = ctx->key (fingerprint, err, false);
+
+  std::stringstream ss;
+  ss << newKey;
+
+  log_debug ("Key: %s", ss.str().c_str());
+  return;
+}
+#endif
+
 #ifdef HAVE_W32_SYSTEM
 ParseController::ParseController(LPSTREAM instream, msgtype_t type):
     m_inputprovider  (new MimeDataProvider(instream,
@@ -483,6 +515,8 @@ ParseController::parse()
                   m_verify_result = ctx->verifyOpaqueSignature(input, output);
                 }
             }
+#else
+(void)allBad;
 #endif
         }
     }
@@ -493,25 +527,14 @@ ParseController::parse()
   bool has_valid_encrypted_checksum = false;
   /* Ensure that the Keys for the signatures are available
      and if it has a valid encrypted checksum. */
-  bool ultimate_keys_queried = false;
   for (const auto sig: m_verify_result.signatures())
     {
       TRACEPOINT;
       has_valid_encrypted_checksum = is_valid_chksum (sig);
 
+#ifndef BUILD_TESTS
       KeyCache::instance ()->update (sig.fingerprint (), protocol);
-
-      if (!ultimate_keys_queried &&
-          (sig.validity() == Signature::Validity::Full ||
-          sig.validity() == Signature::Validity::Ultimate))
-        {
-          /* Ensure that we have the keys with ultimate
-             trust cached for the ui. */
-
-          // TODO this is something for the keycache
-          get_ultimate_keys ();
-          ultimate_keys_queried = true;
-        }
+#endif
       TRACEPOINT;
     }
 
@@ -533,8 +556,12 @@ ParseController::parse()
           const auto key = sig.key();
           if (key.isNull())
             {
+#ifndef BUILD_TESTS
               ss << '\n' << "Cached key:\n" << KeyCache::instance()->getByFpr(
                   sig.fingerprint(), false);
+#else
+              get_and_print_key_test (sig.fingerprint (), protocol);
+#endif
             }
           else
             {
@@ -628,76 +655,3 @@ ParseController::get_attachments() const
       TRETURN std::vector<std::shared_ptr<Attachment> >();
     }
 }
-
-GPGRT_LOCK_DEFINE(keylist_lock);
-/* static */
-std::vector<Key>
-ParseController::get_ultimate_keys()
-{
-  TSTART;
-  static bool s_keys_listed;
-  static std::vector<Key> s_ultimate_keys;
-  gpgol_lock (&keylist_lock);
-  if (s_keys_listed)
-    {
-      gpgol_unlock (&keylist_lock);
-      TRETURN s_ultimate_keys;
-    }
-  log_debug ("%s:%s: Starting keylisting.",
-             SRCNAME, __func__);
-  auto ctx = std::unique_ptr<Context> (Context::createForProtocol (OpenPGP));
-  if (!ctx)
-    {
-      /* Maybe PGP broken and not S/MIME */
-      log_error ("%s:%s: broken installation no ctx.",
-                 SRCNAME, __func__);
-      gpgol_unlock (&keylist_lock);
-      TRETURN s_ultimate_keys;
-    }
-  ctx->setKeyListMode (KeyListMode::Local);
-  Error err;
-  TRACEPOINT;
-  if ((err = ctx->startKeyListing ()))
-    {
-      log_error ("%s:%s: Failed to start keylisting err: %i: %s",
-                 SRCNAME, __func__, err.code (), err.asString());
-      gpgol_unlock (&keylist_lock);
-      TRETURN s_ultimate_keys;
-    }
-  TRACEPOINT;
-  while (!err)
-    {
-      const auto key = ctx->nextKey(err);
-      if (err || key.isNull())
-        {
-          TRACEPOINT;
-          break;
-        }
-      if (key.isInvalid ())
-        {
-          log_debug ("%s:%s: skipping invalid key.",
-                     SRCNAME, __func__);
-          continue;
-        }
-      for (const auto uid: key.userIDs())
-        {
-          if (uid.validity() == UserID::Validity::Ultimate &&
-              uid.id())
-            {
-              s_ultimate_keys.push_back (key);
-              log_debug ("%s:%s: Adding ultimate uid.",
-                         SRCNAME, __func__);
-              log_data ("%s:%s: Added uid %s.",
-                        SRCNAME, __func__, uid.id());
-              break;
-            }
-        }
-    }
-  TRACEPOINT;
-  log_debug ("%s:%s: keylisting done.",
-             SRCNAME, __func__);
-
-  s_keys_listed = true;
-  gpgol_unlock (&keylist_lock);
-  TRETURN s_ultimate_keys;
-}
