@@ -50,6 +50,7 @@
 #define ID_CMD_PRINT_DECRYPTED 34
 #define ID_GET_IS_CRYPTO_MAIL 35
 #define ID_CMD_OPEN_CONTACT_KEY 36
+#define ID_CMD_FILE_CLOSE 37
 
 #define ID_BTN_DECRYPT           IDI_DECRYPT_16_PNG
 #define ID_BTN_DECRYPT_LARGE     IDI_DECRYPT_48_PNG
@@ -88,4 +89,6 @@ HRESULT get_is_crypto_mail (LPDISPATCH ctrl, VARIANT *result);
 HRESULT print_decrypted (LPDISPATCH ctrl);
 /* Open key configuration for a contact */
 HRESULT open_contact_key (LPDISPATCH ctrl);
+/* An explorer is closed by File->Close */
+HRESULT override_file_close ();
 #endif
